package com.example.qualitychecker.service;

import androidx.annotation.NonNull;

import com.example.qualitychecker.apiservice.ApiService;
import com.example.qualitychecker.apiservice.ApiServiceFactory;
import com.example.qualitychecker.models.InspectionItemCheckPoint;
import com.example.qualitychecker.models.InspectionItemDefect;
import com.example.qualitychecker.models.InspectionReport;
import com.example.qualitychecker.models.SaveReportCallback;
import com.example.qualitychecker.store.Store;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Objects;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class InspectionReportService {

    private static InspectionReportService instance;
    private ApiService apiService;
    private List<String> defectTypes;
    private List<String> dimensionTypes;
    private List<String> reportResult;
    private Store store;

    // singleton instance
    public synchronized static  InspectionReportService getInstance(){
        if( instance == null ){
            instance = new InspectionReportService();
        }
       return instance;
    }

    //  initialize api service
    {
        apiService = ApiServiceFactory.getApiService();
        defectTypes  = Arrays.asList( "CRITICAL","MINOR","MAJOR" );
        dimensionTypes = Arrays.asList( "LENGTH", "WIDTH", "DROP" );
        reportResult = Arrays.asList("PASSED","FAILED");
        store = Store.getInstance();
    }

     public void isUserAuthenticated( String username,
                                      String password,
                                      String[] roles,
                                      Callback<Boolean> callback){


         apiService.isUserAuthenticated(username, password, roles).enqueue(
                 new Callback<Boolean>() {
                     @Override
                     public void onResponse(@NonNull Call<Boolean> call, @NonNull Response<Boolean> response) {
                         if (response.isSuccessful()) {
                             callback.onResponse(call, response);
                         } else {
                             // Handle error
                             callback.onFailure(call, new Throwable("Failed to authenticate user"));
                         }
                     }
                     @Override
                     public void onFailure(@NonNull Call<Boolean> call, @NonNull Throwable t) {
                         // Handle failure
                         callback.onFailure(call, t);
                     }
         });
     }

     public List<String> getDefectTypes(){
        return defectTypes;
     }
     public List<String> getDimensionTypes(){return dimensionTypes;}

    public List<String> getReportResult() {
        return reportResult;
    }

    public void setReportResult(List<String> reportResult) {
        this.reportResult = reportResult;
    }

    public void saveReport(InspectionReport inspectionReport , SaveReportCallback callback){

        if( Objects.nonNull( inspectionReport ) && inspectionReport.getItems().get(0).getSku() != null && inspectionReport.getItems() != null &&
                ! inspectionReport.getItems().isEmpty() &&
                ! inspectionReport.getItems().get(0).getCheckPoints().isEmpty() ){
            // populate defects in cp
            for( InspectionItemCheckPoint itemCp : inspectionReport.getItems().get(0).getCheckPoints() ){
                InspectionItemDefect defect = new InspectionItemDefect();
                defect.setDefectSeverity(itemCp.getDefectSeverity());
                defect.setTitle( itemCp.getDefectTitle() );
                itemCp.setDefect( defect );
            }
            // post object
            apiService.saveInspectionReport( inspectionReport ).enqueue(
                    new Callback<InspectionReport>() {
                        @Override
                        public void onResponse(Call<InspectionReport> call, Response<InspectionReport> response) {
                            if (response.isSuccessful()) {
                                callback.onSuccess();
                            } else {
                                callback.onFailure(new Exception("API call failed with status code: " + response.code()));
                            }
                        }
                        @Override
                        public void onFailure(Call<InspectionReport> call, Throwable t) {
                            callback.onFailure( t );
                        }
                    }
            );
        } else {
            callback.onFailure( new Exception("Please Fill the required Fields..") );
        }
     }
}